@@ -555,12 +555,31 @@ static void wispr_portal_browser_reply_cb(struct connman_service *service,
 					const char *error, void *user_data)
 {
 	struct connman_wispr_portal_context *wp_context = user_data;
+	struct connman_wispr_portal *wispr_portal;
+	int index;
 
 	DBG("");
 
 	if (!service || !wp_context)
 		return;
 
+	/*
+	 * No way to cancel this if wp_context has been freed, so we lookup
+	 * from the service and check that this is still the right context.
+	 */
+	index = __connman_service_get_index(service);
+	if (index < 0)
+		return;
+
+	wispr_portal = g_hash_table_lookup(wispr_portal_list,
+					GINT_TO_POINTER(index));
+	if (!wispr_portal)
+		return;
+
+	if (wp_context != wispr_portal->ipv4_context &&
+		wp_context != wispr_portal->ipv6_context)
+		return;
+
 	if (!authentication_done) {
 		wispr_portal_error(wp_context);
 		free_wispr_routes(wp_context);
@@ -568,7 +587,7 @@ static void wispr_portal_browser_reply_cb(struct connman_service *service,
 	}
 
 	/* Restarting the test */
-	__connman_wispr_start(service, wp_context->type);
+	__connman_service_wispr_start(service, wp_context->type);
 }
 
 static void wispr_portal_request_wispr_login(struct connman_service *service,
@@ -972,6 +991,7 @@ int __connman_wispr_start(struct connman_service *service,
 
 void __connman_wispr_stop(struct connman_service *service)
 {
+	struct connman_wispr_portal *wispr_portal;
 	int index;
 
 	DBG("service %p", service);
@@ -983,7 +1003,22 @@ void __connman_wispr_stop(struct connman_service *service)
 	if (index < 0)
 		return;
 
-	g_hash_table_remove(wispr_portal_list, GINT_TO_POINTER(index));
+	wispr_portal = g_hash_table_lookup(wispr_portal_list,
+					GINT_TO_POINTER(index));
+	if (!wispr_portal)
+		return;
+
+	if (wispr_portal->ipv4_context) {
+		if (service == wispr_portal->ipv4_context->service)
+			g_hash_table_remove(wispr_portal_list,
+					GINT_TO_POINTER(index));
+	}
+
+	if (wispr_portal->ipv6_context) {
+		if (service == wispr_portal->ipv6_context->service)
+			g_hash_table_remove(wispr_portal_list,
+					GINT_TO_POINTER(index));
+	}
 }
 
 int __connman_wispr_init(void)
